"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const https = require("https");
const throwback_1 = require("throwback");
const node_1 = require("popsicle/dist/node");
const popsicle_transport_http_1 = require("popsicle-transport-http");
const path_1 = require("path");
const fs_1 = require("fs");
const index_1 = require("./index");
const unsafeFetch = node_1.toFetch(throwback_1.compose([
    popsicle_transport_http_1.transport({
        rejectUnauthorized: false
    })
]), node_1.Request);
describe("server address", () => {
    describe("server function", () => {
        const server = new index_1.ServerAddress(handler);
        it("should throw when retrieving a url and not listening", () => {
            expect(() => server.url("/foo")).toThrow(/Server is not listening/);
        });
        describe("listening", () => {
            beforeEach(() => server.listen());
            afterEach(() => server.close());
            it("should resolve empty paths", () => {
                expect(server.url()).toMatch(/^http:\/\/127\.0\.0\.1:\d+$/);
            });
            it("should support handler functions", () => __awaiter(void 0, void 0, void 0, function* () {
                const res = yield node_1.fetch(server.url("/foo"));
                expect(yield res.text()).toEqual("GET /foo");
            }));
            it("should close immediately", () => __awaiter(void 0, void 0, void 0, function* () {
                const url1 = server.url("/foo");
                const url2 = server.url("/bar");
                const res = yield node_1.fetch(url1);
                expect(yield res.text()).toEqual("GET /foo");
                server.close();
                yield expect(node_1.fetch(url2)).rejects.toThrow(/Unable to connect/);
            }));
        });
    });
    it("should support http servers", () => __awaiter(void 0, void 0, void 0, function* () {
        const server = new index_1.ServerAddress(http.createServer(handler));
        server.listen();
        const res = yield node_1.fetch(server.url("/foo"));
        expect(yield res.text()).toEqual("GET /foo");
        server.close();
    }));
    it("should support https servers", () => __awaiter(void 0, void 0, void 0, function* () {
        const server = new index_1.ServerAddress(https.createServer({
            key: fs_1.readFileSync(path_1.join(__dirname, "../fixtures/key.pem")),
            cert: fs_1.readFileSync(path_1.join(__dirname, "../fixtures/key-cert.pem"))
        }, handler));
        server.listen();
        const res = yield unsafeFetch(server.url("/foo"));
        expect(yield res.text()).toEqual("GET /foo");
        server.close();
    }));
    it("should not listen or close an already listening server", () => __awaiter(void 0, void 0, void 0, function* () {
        const app = http.createServer(handler).listen(0);
        const originalPort = app.address().port;
        const server = new index_1.ServerAddress(app);
        server.listen();
        expect(app.address().port).toEqual(originalPort);
        const res = yield node_1.fetch(server.url("/foo"));
        expect(yield res.text()).toEqual("GET /foo");
        server.close();
        expect(app.address()).toEqual(null);
    }));
    it("should support callback on `listen()`", done => {
        const server = new index_1.ServerAddress(http.createServer(handler));
        server.listen(() => {
            server.close(done);
        });
    });
    it("should support callback on `listen()` when already listening", done => {
        const server = new index_1.ServerAddress(http.createServer(handler).listen(0));
        server.listen(() => {
            server.close(done);
        });
    });
});
/**
 * Create a http request handler.
 */
function handler(req, res) {
    res.write(req.method + " " + req.url);
    res.end();
}
//# sourceMappingURL=index.spec.js.map